// program imgdump
// usage -- %imgdump input > STDOUT

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#define INTEL

#define HiPic_Header_Size 64

struct HiPic_Header
{
	char					head[2];
	unsigned short			comment_length;
	unsigned short			width;
	unsigned short			height;
	unsigned short			x_offset;
	unsigned short			y_offset;
	unsigned short			type;
	char					reserved[63 - 13];
	char					*comment;
};
typedef struct HiPic_Header Header;

unsigned short	*data;

static void	Error(msg)
char		*msg;
{
	fputs(msg,stderr); fputc('\n',stderr); exit(1);
}

/*----------------------------------------------------------------------*/
int read_hipic(fname, h)
char	*fname;
Header	*h;
{
		short		x_size, y_size;
		int		j;
		FILE	*fi;

//open input files
		if((fi = fopen(fname,"rb")) == NULL)
		{
			printf("can not open %s for input\n", fname);
			free(h->comment); 
			return(-10);
		}
// read comment and image data from file1
		if (fread(h, sizeof(char), HiPic_Header_Size, fi) != HiPic_Header_Size)
		{
			fprintf(stderr, "EOF in %s (header)\n", fname);
			free(h->comment); 
			return(-1);
		}
		if (strncmp(h->head, "IM", 2))
		{
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "This is not HiPIc image file.\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-11);
		}

		h->comment = (char *) malloc(h->comment_length + 1);
		if (h->comment == NULL)
		{
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "Connot allocate memory (HiPic_Header.comment)\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-12);
		}
		if (fread(h->comment, sizeof(char), h->comment_length, fi) != h->comment_length)
		{ 
			fclose(fi); 
			free(h->comment); 
			fprintf(stderr, "EOF found (comment)\n"); 
			fprintf(stderr, "File name = %s\n", fname); 
			return(-13);
		}

		h->comment[h->comment_length] = '\0';

		x_size = h->width;
		y_size = h->height;

		data = (unsigned short *) malloc(x_size * y_size * sizeof(unsigned short));
		if ((j = fread(data, sizeof(unsigned short), x_size*y_size, fi)) != x_size*y_size)
		{
			fclose(fi);
			free(h->comment); 
			fprintf(stderr, " Error reading %s at %d\n", fname, j);
			fprintf(stderr, "File name = %s\n", fname); 
			return(1);
		}

// check for correct reading of HiPic file
//printf("-------------------------------\n");
//printf("%s\n", fname);
//printf("%s\n", h->head);
//printf("%d\n", h->comment_length);
//printf("%d\n", h->width);
//printf("%d\n", h->height);
//printf("%d\n", h->x_offset);
//printf("%d\n", h->y_offset);
//printf("%d\n", h->type);
//printf("%s\n", h->reserved);
//printf("%s\n", h->comment);
//printf("-------------------------------\n");

		fclose(fi);
		return(0);
	}

/*----------------------------------------------------------------------*/

int	main(argc,argv)
int	argc;
char	**argv;
{
	int		i, j, k;
	short	Nx, Ny;
	unsigned short	data_max, data_min;


	Header	h;

	if (argc!=2) Error("usage : imgdump input (> STDOUT)");

	if ((i = read_hipic(argv[1], &h)) != 0){
		printf("something wrong -- return value is %d", i);
		exit(-1);
	}

	Nx = h.width;
	Ny = h.height;

	data_max = 0;
	data_min = 32000;
	for (i=0; i<Nx*Ny;i++){
		if(data_max<*(data+i)) data_max=*(data+i);
		if(data_min>*(data+i)) data_min=*(data+i);
	}

	printf("#P2\n");
	printf("#%d %d\n",Nx, Ny);
	printf("#%d\n",data_max);
	j=0;
	for(k=0;k<Ny;++k){
		for (i=0; i<Nx;i++){
			printf("%d ",*(data+j));
		j=j+1;
		}
		printf("\n");
	}

	free(h.comment);
	return 0;
}
